# Make command-line Options for Optiboot, Optiboot-Mega0
# Permit commands like "make TARGET=atmega4809 TIMEOUT=8" to pass the
# appropriate parameters ("-DWDTTIME=8") to gcc

ifdef PRODUCTION
	ifneq ($(PRODUCTION), 0)
		VERSION_CMD = -DPRODUCTION=1
	endif
endif

# Target options

HELPTEXT += "Option TARGET=mcu            - set target MCU, default is attiny1627\n"
ifndef TARGET
	TARGET = attiny1627
endif

# Build Options

HELPTEXT += "Option CUSTOM_VERSION=nn     - set a customer version number\n"
ifdef CUSTOM_VERSION
	ifneq ($(CUSTOM_VERSION), 0)
		VERSION_CMD = -DOPTIBOOT_CUSTOMVER=$(CUSTOM_VERSION)
	else
		VERSION_CMD = -DPRODUCTION=1
	endif
endif

HELPTEXT += "Option NO_APP_SPM=1          - disallow application call of do_spm\n"
ifdef NO_APP_SPM
	ifneq ($(NO_APP_SPM), 0)
		APPSPM_CMD = -DAPP_NOSPM=1
	endif
endif

# CPU Options

HELPTEXT += "Option TIMEOUT=n             - set WDT to 1, 2, 4, or 8 seconds\n"
ifdef TIMEOUT
	TIMEOUT_CMD = -DWDTTIME=$(TIMEOUT)
endif

HELPTEXT += "Option RESETPIN=0/1          - change RESET pin behavior\n"
ifdef RESETPIN
	RESETPIN_CMD = -DRSTPIN=$(RESETPIN)
endif

HELPTEXT += "Option ENTRYCOND=n           - set entry conditions\n"
ifdef ENTRYCOND
	ENTRYCOND_CMD = -DENTRYCOND_REQUIRE=$(ENTRYCOND)
endif

HELPTEXT += "Option AVR_FREQ=<n>          - Clock rate of AVR CPU\n"
ifdef AVR_FREQ
	FCPU_CMD = -DF_CPU=$(AVR_FREQ)
else
	FCPU_CMD = -DF_CPU=16500000
endif


CPU_OPTIONS = $(RESETPIN_CMD) $(TIMEOUT_CMD) $(FCPU_CMD) $(ENTRYCOND_CMD)
COMMON_OPTIONS = $(APPSPM_CMD) $(VERSION_CMD)
